function lead_roi_lut = lead_to_ROI(roi_lut, proj_coords, pial_surf, envelope_surf, roi_radius, varargin)
% lead_to_ROI 
%
% Usage: lead_to_ROI = lead_to_ROI(roi_lut, proj_coords, pial_surf, envelope_surf, roi_radius, working_dir, ...)
%
% Inputs:
%   roi_lut         - output from grow_ROIs: ROI table with columns: vertex, ROIC_mesh_ndx, d
%   proj_coords     - output from proj_coords: Table with xyz coordiantes of every electrode
%   pial_surf       - A struct with faces and vertices fields representing the pial surface mesh
%   evenlope_surf   - A struct with faces and vertices fields representing the cortical envelope mesh
%   roi_radius      - Radius of ROIs (geodesic, mm)
%   
% Optional Input:
%   working_dir     - directory where intermediate files and output files will be saved
%   
%
% Input key-values:
%   log_file        - path to log file. 
%   contact_radius  - radius to model electrode as disk on cortical envelope. Default 1.5 (mm).
%
% Output: 
%   lead_roi_lut    - table of electrodes' ROIs. This table will be of the same form as
%                     the mr_coords table with new columns: lead_verts, ROIC_verts, which are arrays of
%                     pial surface mesh indices.
%
% File Output:
%   
% Description:
%   Takes XYZ coordinates of electrodes and the subject brain/ROIs and returns the ROIs to 
%   which each electrode maps
%   
%

% Revision History:
%   03/17 - MST 

%  Copyright (C) 2017 Mike Trotta

    % -------------
    %%--- Setup ---
    % -------------
    
    % Input
    ip = inputParser;
    ip.addOptional('working_dir', pwd, @ischar);
    ip.addParameter('log_file', []);
    ip.addParameter('contact_radius', 1.5);
    ip.parse(varargin{:});
    log_file = ip.Results.log_file;
    contact_radius = ip.Results.contact_radius;
    
    % Check input
    required_cols = {'x','y','z'};
    assert(istable(proj_coords) && length(required_cols) == length(intersect(required_cols, proj_coords.Properties.VariableNames)),...
        'proj_coords must be a table with x, y, and z columns');

    required_cols = {'vertex','ROIC_mesh_ndx','d'};
    assert(istable(roi_lut) && length(required_cols) == length(intersect(required_cols, roi_lut.Properties.VariableNames)),...
        'roi_lut must be a table with vertex, ROIC_mesh_ndx, and d columns');
    required_fields = {'faces','vertices'};
    
    if sum(ismember(required_fields, fieldnames(pial_surf))) < 2 || ...
        sum(ismember(required_fields, fieldnames(envelope_surf))) < 2
        error('Pial and evelope surfaces must be structures with .faces and .vertices fields');
    end
    
    d_max = max(roi_lut.d);
    if d_max + 1 < roi_radius
        warning('Given ROI radius (%d) is greater than greatest distance in roi_lut (%d) (radius should match that from grow_ROIs)',...
            roi_radius, d_max);
    end
    
    envelope_surf.vertices = single(envelope_surf.vertices);
    pial_surf.vertices = single(pial_surf.vertices);
    
    % ---------------
    %%--- Mapping ---
    % ---------------
    
    % Map electrode to pial nodes (via sphere model on envelope)
    lead_xyz = proj_coords{:,{'x','y','z'}};
    dural_xyz = envelope_surf.vertices;
    pial_xyz = pial_surf.vertices;
    lead_verts = lead_to_pial(lead_xyz, dural_xyz, pial_xyz, contact_radius, 0);
    n = length(lead_verts);
    
    % Get ROIs within ROI radius of every mesh vertex mapped to this electrode
    do_lookup = @(V) roi_lut{roi_lut.d <= roi_radius & ismember(roi_lut.vertex, V), 'ROIC_mesh_ndx'};
    ROIC_mesh_ndxs = cellfun(do_lookup, lead_verts, 'uniformOutput',0);
    
    lead_roi_lut = proj_coords;
    lead_roi_lut.lead_verts = cellfun(@(x) unique(x,'stable'), lead_verts, 'uniformOutput',0);
    lead_roi_lut.ROIC_verts = cellfun(@(x) unique(x,'stable'), ROIC_mesh_ndxs, 'uniformOutput',0);
end