function localizer_atlas(subj, rootEEGdir, overwrite)
    % localizer_atlas(subj, root) reformats post-projection native afni/freesurfer atlas files into a more useful format
    %
    % Monopolar and bipolar atlas files will be placed in the appropriate locations:
    %
    % Output file format details
    %
    %
    % Resources:
    %     Freesurfer atlas outputs: 
    %         https://surfer.nmr.mgh.harvard.edu/fswiki/FsTutorial/AnatomicalROI/FreeSurferColorLUT
    %     Destrieux parc table:
    %         "Automatic parcellation of human cortical gyri and sulci using standard anatomical nomenclature"
    %         https://www.ncbi.nlm.nih.gov/pmc/articles/PMC2937159/
    %         $FREESURFER_HOME/Simple_surface_labels2009.txt 
    %
    % see also: afni_extract_whereami    
   if nargin < 3, overwrite = 0; end
    
    locDirs = localizer_create_directories(subj, rootEEGdir);
    fname = fullfile(locDirs.atlas, 'atlas_monopolar.csv');
    if exist(fname, 'file') && ~overwrite
        fprintf('Atlas file found, will not overwrite: %s\n', fname);
        return;
    end
    
    element_info = getElementInfo(subj, rootEEGdir);
    
    fprintf('Writing atlas file (2+ minutes...)\n');
    
    fname_leads = fullfile(rootEEGdir, subj, 'tal/leads.csv');
    assert(exist(fname_leads,'file') > 0, 'File not found: %s\n', fname_leads);
    leads = readtable(fname_leads);
    
    xyz     = leads{:,{'x','y','z'}};
    chans   = leads.chanName;
    atlas_table = afni_extract_whereami(...
        'working_dir',  locDirs.working, ...
        'suma_dir',     fullfile(locDirs.fs_subj, 'SUMA'), ...
        'coords',       xyz, ...
        'coord_name',   chans, ...
        'orientation',  'RAI', ...
        'overwrite',    1);
    if isempty(atlas_table)
        error('Atlas error: Problem using AFNI whereami command');
        % Note: Mike has encountered a problem with afni not registering the ORIG template space
        %   Check if this is your problem. This particular problem can be fixed by adding this block
        %   to the SessionAtlases.niml file or your ~/abin/AFNI_atlas_spaces.niml:
        % <TEMPLATE_SPACE
        %   space_name="ORIG"
        %   generic_space="ORIG"
        %   comment="Native space of subject"
        % ></TEMPLATE_SPACE>
    end
    
    new_atlas_table = process_table(atlas_table, chans, subj);
    
    % save
    writetable(new_atlas_table, fname);
    fprintf('written!\n');

    % do again for bipolar xyz?
end

function t = process_table(atlas_table, chans, subj)
    MAX_DIST = 5; 
    
    % the table will have fields: atlas, within_mm, label, (prob_type, code), coord_name, 
    atlases = {'aseg','aparc+aseg','aparc.a2009s+aseg'}; % volume (DK), Desikan-Killiany, and Destrieaux, respectively
    atlas_table = atlas_table(atlas_table.within_mm <= MAX_DIST, {'atlas','within_mm','label', 'coord_name', 'code'});
    
    rows = [];
    warned = 0;
    % for every channel
    for c = 1:numel(chans)

        hwType = element_info(strcmpi(element_info.chanName, chans{c}), :).hardwareType;
        
        
        chan_table = atlas_table(strcmpi(atlas_table.coord_name, chans{c}), :);
        if isempty(chan_table)
            error('Channel not found: %s.\n', chans{c});
        end
        
        % find the nearest lobe
        lobe_table = chan_table(strcmpi(chan_table.atlas, 'aparc.lobesStrict+aseg'), :);
        if ~isempty(lobe_table)
            [~,ndx_min] = min(lobe_table.within_mm);
            loberow = lobe_table(ndx_min(1), :);
            if contains(loberow.label{1}, '-') % lobe is usually ctx-?h-[lobe] but can also be 
                parts = strsplit(loberow.label{1},'-');
                lobe = strjoin(parts(3:end), '-');
            else
                lobe = '';
                keyboard;
            end
        else
            lobe = '';
            if ~warned
                warning('It looks like %s was run without the lobesStrict option; can''t find lobes.', subj);
                warned = 1;
            end
        end
        
        
        
        % for each of the atlases
        for i = 1:numel(atlases)
            atlas = atlases{i};
            atlas_chan_table = chan_table(strcmpi(chan_table.atlas, atlas),:);       
            
            % some subjects (probably older runs) only have _rank atlases,
            % the only difference is that the code has been renumbered from FS by afni 
            % Right now the code isn't used, so this is (for the moment) okay:
            if isempty(atlas_chan_table)
                atlas = [atlas '_rank'];
                atlas_chan_table = chan_table(strcmpi(chan_table.atlas, atlas),:);
            end
            
            % only report the nearest 1 location nearest to this channel for each atlas
            m = min(atlas_chan_table.within_mm);
            atlas_chan_table = atlas_chan_table(atlas_chan_table.within_mm == m, :);
            
            % it is possible that there are >1 equally near locations. For each of them...
            for k = 1:height(atlas_chan_table)
                row = table2struct(atlas_chan_table(k, :));
                
                parts = [];
                s = struct();
                s.chanName  = row.coord_name;
                s.hemisphere= '';
                s.lobe      = lobe;
                s.dist_mm   = row.within_mm;
                s.label     = row.label;
                s.hardwareType = hwType;
                s.class     = '';
                s.label_trim= row.label;
                s.desc      = row.label;
                s.code      = row.code;
                s.atlas     = row.atlas;

                if contains(s.label, 'Right')
                    s.hemisphere = 'rh';
                    s.label_trim = strrep(s.label_trim, 'Right', '');
                elseif contains(s.label, 'Left')
                    s.hemisphere = 'lh';
                    s.label_trim = strrep(s.label_trim, 'Left', '');
                end
                
                switch s.atlas
                    case {'aseg' 'aseg_rank'}
                        
                    case {'aparc+aseg' 'aparc.lobesStrict+aseg' 'aparc+aseg_rank'}
                        parts = strsplit(s.label,'-');
                        
                    case {'aparc.a2009s+aseg' 'aparc.a2009s+aseg_rank'}
                        parts = strsplit(s.label,'_');
                        s.desc = atlasFSTranslate('destrieux','surf', s.label, 1);
                end
                
                if ~isempty(parts)
                    if strcmp(parts{1}, 'ctx')
                        parts = parts(2:end);
                    elseif strcmp(parts{1}, 'wm')
                        parts = parts(2:end);
                    else
                        s.type = 'subcortical';
                    end
                    
                    if strcmp(parts{1}, 'lh')
                        s.hemisphere = 'lh';
                        parts = parts(2:end);
                    elseif strcmp(parts{1}, 'rh')
                        s.hemisphere = 'rh';
                        parts = parts(2:end);
                    end 
                    if strcmp(parts{1}, 'G') && numel(parts) > 2 && strcmp(parts{3}, 'S')
                        s.class = 'gyrus-sulcus';
                        parts = parts(4:end);
                    elseif strcmp(parts{1}, 'S')
                        s.class = 'sulcus';
                        parts = parts(2:end);
                    elseif strcmp(parts{1}, 'G')
                        s.class = 'gyrus';
                        parts = parts(2:end);
                    end
                    s.label_trim = strrep(strjoin(parts,'-'), '_', '-');
                end
                s.label_trim = strip(strip(s.label_trim, 'both','-'), 'both','-');
                
                if isempty(rows)
                    rows = s;
                else
                    rows(length(rows)+1) = s;
                end
            end % end of atlas_chan per-row loop
        end % end of atlas
    end % channel loop
    
    t = struct2table(rows);
    
    % 1) not sure why there are row duplicates
    % 2) unique throws an error on the desc field sometimes. Therefore:
    vars = setdiff(t.Properties.VariableNames, 'desc');
    [~, it] = unique(t(:,vars), 'stable');
    t = t(it, :);
    
end

% If we want to ever add the afni-independent freesurfer-based surface parcellation, base it off of this code:
% bd = braindata('suma_dir', fullfile(locDirs.fs_subj, 'SUMA'));
%         parc_table = [];
%         seg_table = table();
%         for hemisphere = hems
%             hemi_chans_subd     = getLeads(subj, root, 'whichHemi',[hemisphere 'h'], 'hardwareType',{'subdural','micro-subdural'}, 'jacktable',jacktable);
%             hemi_chans_depth    = getLeads(subj, root, 'whichHemi',[hemisphere 'h'], 'hardwareType',{'depth'}, 'jacktable',jacktable);
%             bd.fn_load_surf('hemi',[hemisphere 'h'], 'surfType','pial');
%             bd.fn_load_parc('parcType','a2009s','hemi',[hemisphere 'h']);
%             lut                 = bd.parc.a2009s.lut;
%             lead_roi_lut_hemi   = lead_roi_lut(ismember(lead_roi_lut.chanName, hemi_chans_subd),:);
%             parc_table_hem      = lead_roi_lut_hemi(:, {'chanName'});
% 
%             % Cocjin magic;
%             ind = table();
%             [ind.label ind.node] = util_group2ind(lut.label_name, lut.vert_idx); % break out into individual format
%             parc_table_hem.parc = cellfun(@(nodes) unique(ind.label(ismember(ind.node, nodes))), lead_roi_lut_hemi.lead_verts, 'UniformOutput',false);
%             parc_table = [parc_table; parc_table_hem];
% 
% 
%         end